from datetime import datetime
from django.db.models import Manager
from django.db.models import Q

class BoardManager(Manager):
    """Returns all meetings of the type: Governing board"""

    def get_query_set(self):
        return super(BoardManager, self).get_query_set().filter(group__type__slug='board')
	
class ActiveTermManager(Manager):
    """Returns all currently active terms."""

    def get_query_set(self):
        return super(ActiveTermManager, self).get_query_set().filter(start__lte=datetime.now().date()).filter(Q(end__gte=datetime.now().date())|Q(end__isnull=True))

class ActiveGroupManager(Manager):
    """Returns all currently active groups."""

    def get_query_set(self):
        return super(ActiveGroupManager, self).get_query_set().filter(active=True)

class ApprovedManager(Manager):
    """Returns all currently active groups."""

    def get_query_set(self):
        return super(ApprovedManager, self).get_query_set().filter(draft=False)



from django.db import models
from django.utils.translation import ugettext_lazy as _
from datetime import datetime, date
from django.contrib.localflavor.us.models import PhoneNumberField
from django.contrib.auth.models import User
from markup_mixin.models import MarkupMixin
from committees.managers import BoardManager, ActiveTermManager, ActiveGroupManager, ApprovedManager

from django_extensions.db.models import TimeStampedModel, TitleSlugDescriptionModel
from eventy.models import EventTime
from photologue.models import ImageModel, Photo
from simple_history.models import HistoricalRecords

class GroupType(TitleSlugDescriptionModel):
    '''Group type model.

    Allows catering the group type to how orgs are organized into governing groups.
    
    e.g. Governing board, Ad-hoc committee, Congregation'''
    order = models.IntegerField(_('Order'), max_length=2)

    class Meta:
        verbose_name = _('Group type')
        verbose_name_plural = _('Group types')
        ordering = ('order', 'title',)

    def __unicode__(self):
        return self.title

    @models.permalink
    def get_absolute_url(self):
        return ('cm-group-type-detail', None, {'slug': self.slug})

class Group(TimeStampedModel, TitleSlugDescriptionModel):
    '''Group model.

    Manages the various types of governing groups in an organization. (e.g. Governing Board, Music Committee, etc...)
    '''

    order=models.IntegerField(_('Order'), max_length=2, help_text='Used to order groups in a list. Set all ranks the same to sort by alpha.')
    type=models.ForeignKey(GroupType)
    active=models.BooleanField(_('Active'), default=True)
    formed_on=models.DateField(_('Formed on'), blank=True, null=True)
    disbanded_on=models.DateField(_('Disbanded on'), blank=True, null=True)
    adhoc=models.BooleanField(_('Ad-hoc'), default=False, help_text='Is this is an ad-hoc group?')
    members=models.ManyToManyField('Person', related_name='members', blank=True, null=True, help_text='Non-term limited members of the group.')
    past_members=models.ManyToManyField('Person', related_name='past_members', blank=True, null=True, help_text='Non-term limited members who have left the group')
    #deceased_members = models.ManyToManyField('Person', related_name='deceased_members', blank=True, null=True, help_text='Non-term limited members who have passed away')
    special_title=models.CharField(_('Special title'), max_length=255, blank=True, null=True, help_text='If the auto-generated name for the group makes no sense.')
    ex_officio = models.BooleanField(_('Ex-Officio'), default=False, help_text='Does this group fall under the ex-officio officer rules?')

    objects=models.Manager()
    active_objects=ActiveGroupManager()

    def __init__(self, *args, **kwargs):
        super(Group, self).__init__(*args, **kwargs)
        self._eo_members = []

    class Meta:
        verbose_name = _('Group')
        verbose_name_plural = _('Groups')

    def __unicode__(self):
        if self.special_title:
            return u'%s' % self.special_title
        else:
            return u'%s %s' % (self.title, self.type)

    @models.permalink
    def get_absolute_url(self):
        return ('cm-group-detail', None, {'slug': self.slug})

    def save(self, *args, **kwargs):
        if self.disbanded_on and self.active:
            self.active = False
        super(Group, self).save(*args, **kwargs)

    @property
    def current_terms(self):
        return Term.active_objects.filter(group=self)

    @property
    def past_terms(self):
        objects=[]
        for t in Term.objects.all().filter(group=self):
            if not t in Term.active_objects.all().filter(group=self):
                objects.append(t)
        return objects

    @property
    def exofficio_members(self):
        if not self._eo_members:
            if self.ex_officio:
                for o in Office.objects.filter(ex_officio=True):
                    if o.current and (o.current not in self.current_terms):
                        self._eo_members.append(o.current)
        return self._eo_members

class GroupPhoto(ImageModel):
    '''Group photo model.

    Represents a photo of a governing group at a specific time.'''
    group=models.ForeignKey(Group)
    year=models.IntegerField(_('Year'), max_length=4)
    caption=models.TextField(_('Caption'), blank=True, null=True)

    class Meta:
        verbose_name = _('Group photo')
        verbose_name_plural = _('Group photos')
        get_latest_by='year'

    def __unicode__(self):
        return u'%s photo of %s' % (self.year, self.group)
    
class Office(TimeStampedModel, TitleSlugDescriptionModel):
    '''
    Office model.

    Holds all the various office types, mostly just drawn from django-extension 
    mixins. Designed to take care of pres, vp, sec, tres type positions.

    '''
    group=models.ForeignKey(Group)
    order=models.IntegerField(_('Office order'), max_length=2)
    ex_officio=models.BooleanField(_('Ex-Officio'), default=False,
    help_text='An ex-officio office is, by default, a member of all committees, unless stated.')
    
    class Meta:
        verbose_name = _('Office')
        verbose_name_plural = _('Offices')
        ordering = ('order', 'group',)

    def __unicode__(self):
        return u'%s of %s' % (self.title, self.group)

    @property
    def previous(self):
        current = past = None
        terms = self.term_set.all().order_by('-end')
        i=1
        for t in terms:
            try:
                past = terms[i]
            except:
                return None
            if (t.person != past.person) and not t.active:
                return past
            i+=1
        return self.term_set.all().order_by('-end')[1]

    @property
    def current(self):
        terms =[]
        for t in self.term_set.all().order_by('-end'):
            if t.active:
                terms.append(t)
        if len(terms) == 1:
            return terms[0]
        else:
            for t in terms:
                t.office.title = "Co-"+t.office.title
            return terms

class Term(TimeStampedModel):
    group = models.ForeignKey(Group)
    start = models.DateField(_('Start'))
    end = models.DateField(_('End'), blank=True, null=True)
    office = models.ForeignKey(Office, blank=True, null=True)
    alternate = models.BooleanField(_('Alternate'), default=False)
    person=models.ForeignKey('Person', blank=True, null=True)
    
    objects = models.Manager()
    board_members = BoardManager()
    active_objects = ActiveTermManager()

    class Meta:
        verbose_name = _('Term')
        verbose_name_plural = _('Terms')
        ordering = ('-office','start',)
        get_latest_by = 'start'
   
    @property
    def active(self):
        status=False
        if self.start <= date.today():
            if self.end:
                if self.end >= date.today():
                    status = True
            else:
                status=True
        return status
    
    @property
    def length(self):
        if self.end: return (self.end.year - self.start.year)
        else: return None
    
    @property
    def tenure(self):
        tenure = 0
        terms = Term.objects.filter(person=self.person)
        for t in terms:
            if t.length:
                tenure += self.length
            elif not t.end:
                tenure += datetime.now().year - self.start.year

        return tenure

    @property
    def officer(self):
        if self.office: return True
        else: return False

    def __unicode__(self):
        if self.officer: desc=self.office
        else: desc=u'%s member' % (self.group)
        
        if self.alternate: str=u'%s - alternate (%s)' % (self.person, self.start.year)
        else: str=u'%s - %s (%s)' %(self.person, desc, self.start.year)
        return str

    @models.permalink
    def get_absolute_url(self):
        if self.office: return ('cm-term-detail', None, {'slug': self.group.slug, 'office_slug': self.office.slug, })
        else: return None

class Person(models.Model):
    """Person model."""
    GENDER_CHOICES = (
        (1, 'Male'),
        (2, 'Female'),
    )
    first_name = models.CharField(_('First name'), blank=True, max_length=100)
    middle_name = models.CharField(_('Middle name'), blank=True, max_length=100)
    last_name = models.CharField(_('Last name'), blank=True, max_length=100)
    title = models.CharField(_('Title'), blank=True, max_length=100)
    slug = models.SlugField(_('Slug'), unique=True)
    user = models.ForeignKey(User, blank=True, null=True, help_text='If the person is an existing user of your site.')
    gender = models.PositiveSmallIntegerField(_('Gender'), choices=GENDER_CHOICES, blank=True, null=True)
    member = models.BooleanField(_('Member'), default=True, help_text='Is this person a member of the organization?')
    phone = PhoneNumberField(_('Phone'), blank=True, null=True)
    email = models.EmailField(_('Email'), blank=True, null=True)
    photo = models.ForeignKey(Photo, blank=True, null=True)
    bio = models.TextField(_('Biography'), blank=True, null=True)

    class Meta:
        verbose_name = _('person')
        verbose_name_plural = _('people')
        ordering = ('last_name', 'first_name',)

    def __unicode__(self):
        return u'%s' % self.full_name

    @property
    def full_name(self):
        return u'%s %s' % (self.first_name, self.last_name)

    @property
    def on_board(self):
        for t in self.term_set.all():
            if t.group.order == 10:
                return True

    @models.permalink
    def get_absolute_url(self):
        return ('cm-person-detail', None, {'slug': self.slug})


class Meeting(MarkupMixin, TimeStampedModel, EventTime):
    '''
    Meeting model.

    A general meeting model. 
    '''
    group = models.ForeignKey(Group)
    agenda = models.TextField(_('Agenda'), blank=True, null=True)
    rendered_agenda = models.TextField(_('Rendered agenda'), blank=True, null=True)
    business_arising=models.TextField(_('Business arising'), blank=True, null=True)

    def __init__(self, *args, **kwargs):
        super (Meeting, self).__init__(*args, **kwargs)
        self._next = None
        self._previous = None

    class Meta:
        verbose_name = _('Meeting')
        verbose_name_plural = _('Meetings')
        ordering = ('start',)

    class MarkupOptions:
        source_field = 'agenda'
        rendered_field = 'rendered_agenda'

    def __unicode__(self):
        return u'%s meeting - %s' % (self.group, self.start)

    @models.permalink
    def get_absolute_url(self):
        return ('cm-meeting-detail', (), {'slug': self.group.slug, 'year': self.start.year, 'month': self.start.month, })

    def get_next_meeting(self):
        """Determines the next meeting"""

        if not self._next:
            try:
                qs = Meeting.objects.filter(event__calendar=self.event.calendar).exclude(id__exact=self.id)
                meeting= qs.filter(start__gte=self.start).order_by('start')[0]
            except (Meeting.DoesNotExist, IndexError):
                meeting = None
            self._next = meeting 

        return self._next

    def get_previous_meeting(self):
        """Determines the previous meeting"""

        if not self._previous:
            try:
                qs = Meeting.objects.filter(event__calendar=self.event.calendar).exclude(id__exact=self.id)
                meeting= qs.filter(start__lte=self.start).order_by('-start')[0]
            except (Meeting.DoesNotExist, IndexError):
                meeting = None
            self._previous = meeting 

        return self._previous

class Minutes(MarkupMixin, TimeStampedModel):
    meeting = models.ForeignKey(Meeting)
    call_to_order = models.TimeField(_('Call to order'), blank=True, null=True)
    members_present = models.ManyToManyField(Term, related_name='members_present')
    others_present = models.ManyToManyField(Person, related_name='others_present')
    members_present_new = models.ManyToManyField(Person, related_name='meeting_members')
    guests_present = models.ManyToManyField(Person, related_name='meeting_guests', blank=True, null=True)
    content = models.TextField(_('Content'))
    rendered_content = models.TextField(_('Rendered content'), blank=True, null=True, editable=False)
    adjournment = models.TimeField(_('Adjournment'), blank=True, null=True)
    signed = models.ForeignKey(Person, related_name="signed_by")
    signed_date = models.DateField(_('Signed date'), default=datetime.now())
    draft = models.BooleanField(_('Draft'), default=True)

    objects = models.Manager()
    approved_objects = ApprovedManager()
    history = HistoricalRecords()

    class Meta:
        verbose_name = _('Minutes')
        verbose_name_plural = _('Minutes')
        ordering = ('meeting',)

    class MarkupOptions:
        source_field = 'content'
        rendered_field = 'rendered_content'

    def __unicode__(self):
        return u'Minutes from %s' % (self.meeting)

    @models.permalink
    def get_absolute_url(self):
        return ('cm-minutes-detail', (), {'slug':self.meeting.meeting.group.slug, 'year': self.meeting.meeting.start.year, 'month': self.meeting.meeting.event.start.month, })

class Attachment(TimeStampedModel):
    upload_to = lambda inst, fn: 'attach/%s/%s/%s' % (datetime.now().year, inst.minutes.meeting.group.slug, fn)

    minutes = models.ForeignKey(Minutes, related_name='attachments')
    attachment = models.FileField(upload_to=upload_to)
    title = models.CharField(_('Title'), max_length=255)
    description = models.TextField(_('Description'), blank=True, null=True)

    class Meta:
        ordering = ('-minutes', 'id')

    def __unicode__(self):
        return u'%s: %s' % (self.minutes, self.title)

    @property
    def filename(self):
        return self.attachment.name.split('/')[-1]

    @property
    def content_type_class(self):
        mt = mimetypes.guess_type(self.attachment.path)[0]
        if mt:
            content_type = mt.replace('/', '_')
        else:
            # assume everything else is text/plain
            content_type = 'text_plain'

        return content_type




from django.contrib import admin
from committees.models import *

class AttachmentInline(admin.TabularInline):
    model = Attachment
    extra = 5
    max_num = 15


class TermAdmin(admin.ModelAdmin):
    list_display = ('person', 'office', 'group', 'start', 'end', 'alternate','is_active',)
    search_fields = ('person','group',)
    list_filter = ( 'person','group',  )

    def is_active(self, object_):
        return object_.active
    is_active.short_description=u'Active?'
    is_active.boolean = True

admin.site.register(Term, TermAdmin)

class MinutesAdmin(admin.ModelAdmin):
    list_display = ('meeting', 'draft', 'call_to_order', 'adjournment', 'signed', 'signed_date',)
    search_fields = ('content','members_present','meeting',)
    filter_horizontal = ('members_present', 'guests_present','members_present_new','guests_present',)
    inlines = [
        AttachmentInline,
    ]

admin.site.register(Minutes, MinutesAdmin)
    
admin.site.register(GroupType)
admin.site.register(GroupPhoto)
admin.site.register(Group)
admin.site.register(Office)
admin.site.register(Person)
admin.site.register(Meeting)



import re
import logging

from django import template
from django.conf import settings
from django.db import models
from committees.models import Group, Office

register = template.Library()

@register.tag
def get_office(parser, token):
    """
    Gets an office if it exists

    Syntax::

    {% get_office [slug] as [var_name] %}

    Example usage::

    {% get_office 'president' as prez %}

    """
    args = token.split_contents()
    argc = len(args)

    try:
        assert argc == 4
    except AssertionError:
        raise template.TemplateSyntaxError('Invalid get_office syntax.')
    # determine what parameters to use
    slug = var_name = None
    if argc == 4:
        t, slug, a, var_name = args
    return GetOfficeNode(slug=slug, var_name=var_name)

class GetOfficeNode(template.Node):
    def __init__(self, slug, var_name):
        self.slug = slug
        self.var_name = var_name

    def render(self, context):
        try:
            self.office = Office.objects.get(slug=self.slug)
        except:
            self.office = None
        context[self.var_name] = self.office
        return ''

class GetGroupsNode(template.Node):
    def __init__(self, status, order, var_name):
        self.var_name = var_name
        self.status = status
        self.order = order

    def render(self, context):
        groups = '' 
        if self.order: q = models.Q(order=self.order)
        else: q = models.Q()

        if self.status == 'active': groups = Group.active_objects.filter(q)
        elif self.status == 'inactive': groups = Group.objects.filter(q).exclude(active=True)
        elif self.status == 'all': groups = Group.objects.filter(q)
        else: raise template.TemplateSyntaxError('Invalid get_committee_groups syntax where order = %s, status = %s and var_name = %s' % (self.order, self.status, self.var_name))

        if groups and len(groups)==1:
            context[self.var_name] = groups[0]
        else:
            context[self.var_name] = groups 
        return ''


@register.tag
def get_committee_groups(parser, token):
    """
    Gets any number of groups ordered by order and places them in a varable.

    Syntax::

    {% get_committee_groups (order [order]|[status]) as [var_name] %}

    Example usage::

    {% get_committee_groups order 10 as boards %}
    {% get_committee_groups inactive order 10 as boards %}
    {% get_committee_groups active as boards %}
    {% get_committee_groups inactive as committees %}
    {% get_comm_groups all as groups %}
    """
    args = token.split_contents()
    argc = len(args)

    try:
        assert argc in (4,5,6)
    except AssertionError:
        raise template.TemplateSyntaxError('Invalid get_committee_groups syntax.')
    # determine what parameters to use
    order = status = var_name = None
    if argc == 4: t, status, a, var_name = args
    elif argc == 5: t, o, order, a, var_name = args
    elif argc == 6: t, status, o, order, a, var_name = args
    return GetGroupsNode(status=status, order=order, var_name=var_name)

class GetGroupNode(template.Node):
    def __init__(self, slug, var_name):
        self.slug = slug
        self.var_name = var_name

    def render(self, context):
        try:
            group = Group.active_objects.get(slug__exact=self.slug)
            context[self.var_name] = group
        except:
            context[self.var_name] = None
        return ''


@register.tag
def get_committee_group(parser, token):
    """
    Gets a a group if it exists

    Syntax::

    {% get_group [slug] as [var_name] %}

    Example usage::

    {% get_group 'governing-board' as board %}

    """
    args = token.split_contents()
    argc = len(args)

    try:
        assert argc == 4
    except AssertionError:
        raise template.TemplateSyntaxError('Invalid get_group syntax.')
    # determine what parameters to use
    slug = var_name = None
    if argc == 4:
        t, slug, a, var_name = args
    return GetGroupNode(slug=slug, var_name=var_name)

class GetMinutesNode(template.Node):
    def __init__(self, meeting, var_name):
        self.meeting=template.Variable(meeting)
        self.var_name = var_name

    def render(self, context):
        try:
            meeting = self.meeting.resolve(context)
            try:
                context[self.var_name] = meeting.minutes_set.all()[0]
                return ''
            except:
                context[self.var_name] = None
                return ''
        except template.VariableDoesNotExist:
            return ''


@register.tag
def get_committee_minutes(parser, token):
    """
    Gets most recent minutes of a meeting if they exist

    Syntax::

    {% get_committee_minutes from [meeting object] as [var_name] %}

    """
    args = token.split_contents()
    argc = len(args)

    try:
        assert argc == 5
    except AssertionError:
        raise template.TemplateSyntaxError('Invalid get_committee_minutes syntax.')
    # determine what parameters to use
    meeting = var_name = None
    if argc == 5:
        t, f, meeting, a, var_name = args
    return GetMinutesNode(meeting=meeting, var_name=var_name)



from django.conf.urls.defaults import *
from django.conf import settings

from committees import views

urlpatterns = patterns('committees.views',
    url (r'^$', view=views.index, name='cm-index', ),
    url (r'^(?P<slug>[-\w]+)/officer/(?P<office_slug>[-\w]+)/$', view=views.term_detail, name='cm-term-detail', ),
    url (r'^(?P<slug>[-\w]+)/officer/(?P<office_slug>[-\w]+)/(?P<start_year>[\d]+)/$', view=views.term_detail, name='cm-term-archive-year', ),
    url (r'^(?P<slug>[-\w]+)/$', view=views.group_detail, name='cm-group-detail', ),
    url (r'^(?P<slug>[-\w]+)/meetings/$', view=views.group_meeting_list, name='cm-group-meeting-list', ),
    url (r'^(?P<slug>[-\w]+)/meetings/(?P<year>[\d]+)/$', view=views.group_meeting_archive_year, name='cm-group-meeting-archive-year', ),
    url (r'^(?P<slug>[-\w]+)/meetings/(?P<year>[\d]+)/(?P<month>[\w]+)/$', view=views.group_meeting_detail, name='cm-meeting-detail', ),
    url (r'^(?P<slug>[-\w]+)/meetings/(?P<year>[\d]+)/(?P<month>[\w]+)/print/$', view=views.group_meeting_detail, name='cm-meeting-print', ),
    
)



from setuptools import setup, find_packages

setup(
    name='django-committees',
    version=__import__('committees').__version__,
    license="BSD",

    install_requires = [
        'django-markup-mixin',
        'django-extensions',
        'simple_history',
        'django-photologue',
        'django-eventy',],

    description='A simple reusable application for managing a small orgs governance in a Django application.',
    long_description=open('README.rst').read(),

    author='Colin Powell',
    author_email='colin@onecardinal.com',

    url='http://github.com/powellc/django-committees',
    download_url='http://github.com/powellc/django-committees/downloads',

    include_package_data=True,

    packages=['committees'],

    zip_safe=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ]
)



from datetime import datetime

from django.template.context import RequestContext
from django.views.generic import list_detail
from django.shortcuts import render_to_response, get_object_or_404

from committees.models import Group, Meeting, Minutes, Term

def index(request):
    objects = Group.active_objects.all().order_by('order')
    past_meetings = Meeting.objects.filter(start__lte=datetime.now())
    return render_to_response('committees/index.html', locals(),
                              context_instance=RequestContext(request))

def group_detail(request, slug):
    object=Group.objects.get(slug=slug)
    members=object.term_set.filter(start__lte=datetime.now(), end__gte=datetime.now())
    meetings=object.meeting_set.filter(start__gte=datetime.now())

    return render_to_response('committees/group_detail.html', locals(),
                              context_instance=RequestContext(request))

def group_meeting_list(request, slug):
    group = Group.objects.get(slug=slug)
    objects = Meeting.objects.filter(group=group)
    return render_to_response('committees/meeting_list.html', locals(),
                  context_instance=RequestContext(request))

def group_meeting_archive_year(request, slug, year):
    group = Group.objects.get(slug=slug)
    objects = Meeting.objects.filter(group=group, start__year=year)
    return render_to_response('committees/meeting_list.html', locals(),
                  context_instance=RequestContext(request))

def group_meeting_detail(request, slug, year, month):
    object = Meeting.objects.get(group__slug=slug, start__year=year, start__month=month)
    if request.user.is_authenticated():
        if request.user.person_set.all()[0].on_board:
            board_member=True
    return render_to_response('committees/meeting_detail.html', locals(),
                  context_instance=RequestContext(request))

 
def minutes_detail(request, slug, year, month):
    meeting = Meeting.objects.get(meeting__group__slug=slug, start__year=year, start__month=month)
    object = Minutes.objects.get(meeting=meeting)
    return render_to_response('committees/minutes_detail.html', locals(),
                  context_instance=RequestContext(request))

def term_detail(request, slug, office_slug, start_year=None):
    if start_year:
        term = Term.objects.get(office__slug=office_slug, group__slug=slug,  start__year=start_year)
    else:
        terms = Term.objects.filter(office__slug=office_slug, group__slug=slug )
        if len(terms) == 1:
            term = terms[0]
    return render_to_response('committees/term_detail.html', locals(),
                  context_instance=RequestContext(request))
